//+------------------------------------------------------------------+
//|                                                    saveTicks.mq5 |
//|                                                        NicolasXu |
//|                                       https://www.noWebsite5.com |
//+------------------------------------------------------------------+
#property copyright "NicolasXu"
#property link      "https://www.noWebsite5.com"
#property version   "1.00"
#include <Arrays\List.mqh>
class Tick: public CObject {
   public:
      datetime timeStamp;
      double   bid;
      double   ask;
      ulong    volume;
      
      // this is constructor
      Tick(datetime _timeStamp, double _bid, double _ask, ulong _volume) {
         this.timeStamp = _timeStamp;
         this.bid = _bid;
         this.ask = _ask;
         this.volume = _volume;
        
      }
};

class TickList: public CList {
   public:
      TickList();
      
      ulong onTickCounter;
      int maxTickNumber;
      bool isNewDate();
      void addLatestTick();
      bool saveTicks();
      void removeTicks();
      void processNewTick();
      void counterAddOne();
     
};
// constructor
TickList::TickList(void){
   this.onTickCounter = 0;
   this.maxTickNumber = 10000; // 10k
}



void TickList::counterAddOne(void){
   this.onTickCounter++;
}

bool TickList::isNewDate() {
   
   int totalTickCount = this.Total();
   
   if(totalTickCount == 0) {
      // empty list is not new day
      return false;
   }
   
   Tick *lastTick = this.GetNodeAtIndex(totalTickCount -1);
   datetime lastTickTimeStamp = lastTick.timeStamp;
   string lastDateString = TimeToString(lastTickTimeStamp, TIME_DATE);
   
   MqlTick current_tick;
   SymbolInfoTick(_Symbol, current_tick);
   datetime currentTickTimeStamp = current_tick.time;
   string thisDateString = TimeToString(currentTickTimeStamp, TIME_DATE);
   
   if(StringCompare(lastDateString, thisDateString) == 0) {
      // same date
      return false;
   } else {
      // new date
      return true;
   }
}

void TickList::addLatestTick(void) {

   MqlTick latest_tick;
   SymbolInfoTick(_Symbol,latest_tick);
   
   
   Tick *theTick = new Tick(latest_tick.time, latest_tick.bid, latest_tick.ask, latest_tick.volume);
   this.Add(theTick);

}

bool TickList::saveTicks() {

   ResetLastError();

   Tick *fstTick = this.GetFirstNode();
   datetime firstTickTimeStamp = fstTick.timeStamp;
   
   string dateStr = TimeToString(firstTickTimeStamp,TIME_DATE);
   
   string fileName = "ticks"+dateStr+".csv"; // e..g: ticks20130418.csv
   int filehandle=FileOpen("tttt\\" + fileName, FILE_READ|FILE_WRITE|FILE_CSV,',', CP_UTF8);
   //C:\Users\nick\AppData\Roaming\MetaQuotes\Tester\158904DFD898D640E9B813D10F9EB397\Agent-127.0.0.1-3000\MQL5\Files\
   
   if(filehandle!=INVALID_HANDLE) {
      
      //Print("FileOpen OK");
      if(FileSeek(filehandle,0,SEEK_END) == false) {
         printf("file seek failed");
      }
      
      int total = this.Total();
      for(int i=0; i < total; i++) {
         Tick *thisTick = tickList.GetNodeAtIndex(i);
         string outputStr = StringFormat( "%i, %G, %G, %i", 
            thisTick.timeStamp, thisTick.bid, thisTick.ask  ,thisTick.volume  );
         FileWrite(filehandle, outputStr );
      }
      FileFlush(filehandle);
      FileClose(filehandle);
      
   } else {
      Print("Operation FileOpen failed, error ",GetLastError());
      return false; 
   }

   return true;
}

void TickList::removeTicks() {

}
void TickList::processNewTick() {

   this.counterAddOne();
   // counter add one
   
   if(this.isNewDate()) {
      if(this.saveTicks()){
         this.Clear();
         printf("total tick count is: %i", this.onTickCounter);
         // reset for new day
         this.onTickCounter = 0;
      } else {
         Print("saving tick error");
      }
      this.addLatestTick();
      return;
   }
   
   this.addLatestTick();
   // 1. add it to list
   
   if(this.Total() >= this.maxTickNumber) {
   // 2. check if total > 10000
      if(this.saveTicks()) {
         // 3. save success
         this.Clear();
         // 4. clear list
      } else {
         Print("saving tick error");
      }
   }
}

TickList *tickList;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- create timer
   EventSetTimer(60);
   tickList = new TickList();
   
   datetime startTime = 1440516982;
   string timeStr = TimeToString(startTime,TIME_DATE|TIME_MINUTES|TIME_SECONDS);
   printf("time string: " + timeStr);
      
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//--- destroy timer
   EventKillTimer();
   if(tickList.saveTicks()){
       tickList.Clear();
       printf("total tick count is: %i", tickList.onTickCounter);
   } else {
      Print("saving tick error");
   }
  
   delete tickList;
      
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick() {
//---
  
   tickList.processNewTick();
   
   
}
//+------------------------------------------------------------------+
//| Timer function                                                   |
//+------------------------------------------------------------------+
void OnTimer() {
//---
   
}
//+------------------------------------------------------------------+
//| Trade function                                                   |
//+------------------------------------------------------------------+
void OnTrade(){
//---
   
}
//+------------------------------------------------------------------+
//| TradeTransaction function                                        |
//+------------------------------------------------------------------+
void OnTradeTransaction(const MqlTradeTransaction& trans,
                        const MqlTradeRequest& request,
                        const MqlTradeResult& result) {
//---
   
}
//+------------------------------------------------------------------+
//| Tester function                                                  |
//+------------------------------------------------------------------+
double OnTester(){
//---
   double ret=0.0;
//---

//---
   return(ret);
}
//+------------------------------------------------------------------+
//| TesterInit function                                              |
//+------------------------------------------------------------------+
void OnTesterInit(){
//---
   
}
//+------------------------------------------------------------------+
//| TesterPass function                                              |
//+------------------------------------------------------------------+
void OnTesterPass() {
//---
   
}
//+------------------------------------------------------------------+
//| TesterDeinit function                                            |
//+------------------------------------------------------------------+
void OnTesterDeinit() {
//---
   
}
//+------------------------------------------------------------------+

void saveTicks(int tickCountToSave) {
   

   

}